import pandas as pd
import re
import os


def pg2bq(sql):
    """
    Translate a SQL query string from PostgreSQL dialect to Big Query SQL dialect
    Please note that this has only been tested with queries from this repository so your mileage may vary.

    Returns:
        dataframe:
    """

    # translate SPLIT_PART to SPLIT
    """
    PostgreSQL:
    split_part(string text, delimiter text, field int)
    Split string on delimiter and return the given field (counting from one)
    
    BigQuery: 
    SPLIT(value[, delimiter])
    Splits value using the delimiter argument. Return type: ARRAY of type STRING or ARRAY of type BYTES
    """
    re_pg_split_part = r'split_part\(\s*(.*)\s*,\s*(.*)\s*,\s*([0-9]*)\s*\)'
    re_bq_split = r'SPLIT(\1, \2)[SAFE_ORDINAL(\3)]'
    sql_translated = re.sub(re_pg_split_part, re_bq_split, sql)

    # translate ILIKE to LIKE
    """"
    PostgreSQL: 
    The key word ILIKE can be used instead of LIKE to make the match case-insensitive according to the active locale. 
    This is not in the SQL standard but is a PostgreSQL extension.
    """""
    re_pg_ilike = r'(?P<string>[^\(\s]+)\s+(?P<not>NOT\s+)?(?P<operator>ILIKE)\s+(?P<pattern>(?:(?P<quote>[\"\'])' \
                  r'(?:.)*?(?P=quote))|(?:[^\)\s]+))'
    re_bq_like = r'UPPER(\g<string>) \g<not>LIKE UPPER(\g<pattern>)'
    sql_translated = re.sub(re_pg_ilike, re_bq_like, sql_translated)

    # translate Byte to integer
    # after exporting to csv: bytes will be converted into integers
    re_pg_byte = r'(?P<column>[^\(\s]+)\s+(?P<operator>=)\s+(?P<byte>B\'(?P<value>[0-1])\')'
    re_bq_int = r'\g<column> \g<operator> \g<value>'
    sql_translated = re.sub(re_pg_byte, re_bq_int, sql_translated)
    return sql_translated




import os

import numpy as np
import pandas as pd
from .util import read_sql

dirname = os.path.dirname(os.path.abspath(__file__))


def get_sofa_respiration(con) -> pd.DataFrame:
    """
    Return SOFA Respiration: PaO2 / FiO2 (mmHg)

    Simultaneously retrieve PaO2 and the 'nearest' FiO2 from the ventilator or estimated FiO2 based on applied oxygen
    device. Ideally documentation of measurements should be at the same time, but since this is not guaranteed allow an
    offset. Note: in more recent data PaCO2 and PaO2 were documented in kPa instead of mmHg.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    print('Querying SOFA Respiration...')
    filename = './sql/common/pO2_FiO2_estimated.sql'
    sql_filename = os.path.join(dirname, filename)
    with open(sql_filename, 'r') as sql_file:
        sql_sofa_respiration = sql_file.read()
    sofa_respiration = read_sql(sql_sofa_respiration, con)

    print('Processing SOFA Respiration...')
    # remove extreme outliers
    sofa_respiration.loc[(sofa_respiration['fio2'] > 100), 'fio2'] = np.NaN

    # convert FiO2 in % to fraction
    sofa_respiration.loc[(sofa_respiration['fio2'] <= 100) &
                         (sofa_respiration['fio2'] >= 20), 'fio2'] = sofa_respiration['fio2'] / 100

    # remove extreme outliers (FiO2) (possible O2 flow?)
    sofa_respiration.loc[(sofa_respiration['fio2'] > 1), 'fio2'] = np.NaN

    # remove lower outliers, most likely incorrectly labeled as 'arterial' instead of '(mixed/central) venous'
    sofa_respiration.loc[sofa_respiration['pao2'] < 50, 'pao2'] = np.NaN
    sofa_respiration = sofa_respiration.dropna(subset=['pao2'])

    # calculate PF-ratio
    sofa_respiration.loc[:, 'pf_ratio'] = sofa_respiration['pao2'] / sofa_respiration['fio2']

    # calculate SOFA respiration score:
    sofa_respiration.loc[:, 'sofa_respiration_score'] = 0
    sofa_respiration.loc[(sofa_respiration['pf_ratio'] < 400) &
                         (sofa_respiration['pf_ratio'] >= 300), 'sofa_respiration_score'] = 1
    sofa_respiration.loc[(sofa_respiration['pf_ratio'] < 300), 'sofa_respiration_score'] = 2
    sofa_respiration.loc[(sofa_respiration['pf_ratio'] < 200) & (sofa_respiration['pf_ratio'] >= 100) &
                         (sofa_respiration['ventilatory_support'] is True), 'sofa_respiration_score'] = 3
    sofa_respiration.loc[(sofa_respiration['pf_ratio'] < 100) &
                         (sofa_respiration['ventilatory_support'] is True), 'sofa_respiration_score'] = 4

    return sofa_respiration.sort_values(by=['admissionid', 'time']).reset_index(drop=True)


def get_sofa_cardiovascular_meds(con) -> pd.DataFrame:
    """
    Returns SOFA: Cardiovascular - Hypotension, cardiovascular medication: vasopressors and / or inotropes
    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    print('Querying SOFA Cardiovascular: vasopressors/inotropes...')
    filename = './sql/common/vasopressors_inotropes.sql'
    sql_filename = os.path.join(dirname, filename)
    with open(sql_filename, 'r', encoding='utf-8') as sql_file:
        sql_sofa_cardiovascular = sql_file.read()
    sofa_cardiovascular = read_sql(sql_sofa_cardiovascular, con)

    print('Processing SOFA Cardiovascular: vasopressors/inotropes...')
    sofa_cardiovascular_meds = sofa_cardiovascular.groupby(['admissionid', 'itemid', 'item']).agg(
        total_duration=pd.NamedAgg(column='duration', aggfunc='sum'),
        max_gamma=pd.NamedAgg(column='gamma', aggfunc='max')
    ).reset_index()

    # calculate SOFA cardiovascular score:
    sofa_cardiovascular_meds.loc[:, 'sofa_cardiovascular_score'] = 0

    # dopamine (itemid 7179) <= 5 or dobutamine (itemid 7178) any dose
    sofa_cardiovascular_meds.loc[(
                                         ((sofa_cardiovascular_meds['itemid'] == 7179) & (
                                                 sofa_cardiovascular_meds['max_gamma'] <= 5)) |
                                         (sofa_cardiovascular_meds['itemid'] == 7178)
                                 ), 'sofa_cardiovascular_score'] = 2

    # dopamine (itemid 7179) > 5, epinephrine (itemid 6818) <= 0.1, norepinephrine (itemid 7229) <= 0.1
    sofa_cardiovascular_meds.loc[(
                                         ((sofa_cardiovascular_meds['itemid'] == 7179) & (
                                                 sofa_cardiovascular_meds['max_gamma'] > 5) &
                                          (sofa_cardiovascular_meds['max_gamma'] < 15)) |
                                         ((sofa_cardiovascular_meds['itemid'] == 6818) & (
                                                 sofa_cardiovascular_meds['max_gamma'] <= 0.1)) |
                                         ((sofa_cardiovascular_meds['itemid'] == 7229) & (
                                                 sofa_cardiovascular_meds['max_gamma'] <= 0.1))
                                 ), 'sofa_cardiovascular_score'] = 3

    # dopamine (itemid 7179) > 15, epinephrine (itemid 6818) > 0.1, norepinephrine (itemid 7229) > 0.1
    sofa_cardiovascular_meds.loc[(
                                         ((sofa_cardiovascular_meds['itemid'] == 7179) & (
                                                 sofa_cardiovascular_meds['max_gamma'] > 15)) |
                                         ((sofa_cardiovascular_meds['itemid'] == 6818) & (
                                                 sofa_cardiovascular_meds['max_gamma'] > 0.1)) |
                                         ((sofa_cardiovascular_meds['itemid'] == 7229) & (
                                                 sofa_cardiovascular_meds['max_gamma'] > 0.1))
                                 ), 'sofa_cardiovascular_score'] = 4
    return sofa_cardiovascular_meds.sort_values(by='admissionid').reset_index(
        drop=True)


def get_sofa_platelets(con) -> pd.DataFrame:
    """
    Return SOFA Coagulation score.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    ############################################
    print('Querying SOFA Coagulation...')
    # get platelets (thrombocytes)
    filename = './sql/common/platelets.sql'
    sql_filename = os.path.join(dirname, filename)
    with open(sql_filename, 'r') as sql_file:
        sql_sofa_platelets = sql_file.read()
    sofa_platelets = read_sql(sql_sofa_platelets, con)

    # calculate SOFA coagulation score:
    print('Processing SOFA Coagulation...')
    sofa_platelets.loc[:, 'sofa_coagulation_score'] = 0
    sofa_platelets.loc[(sofa_platelets['value'] < 150) &
                       (sofa_platelets['value'] >= 100), 'sofa_coagulation_score'] = 1
    sofa_platelets.loc[(sofa_platelets['value'] < 100) &
                       (sofa_platelets['value'] >= 50), 'sofa_coagulation_score'] = 2
    sofa_platelets.loc[(sofa_platelets['value'] < 50) &
                       (sofa_platelets['value'] >= 20), 'sofa_coagulation_score'] = 3
    sofa_platelets.loc[(sofa_platelets['value'] < 20), 'sofa_coagulation_score'] = 4

    return sofa_platelets.sort_values(by=['admissionid', 'itemid', 'time']).reset_index(drop=True)


def get_sofa_bilirubin(con) -> pd.DataFrame:
    """
    Returns SOFA Liver scores.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    # SOFA: Liver - Bilirubin (µmol/l)
    ##################################
    # get bilirubin
    print('Querying SOFA Liver...')
    filename = './sql/common/bilirubin.sql'
    sql_filename = os.path.join(dirname, filename)
    with open(sql_filename, 'r') as sql_file:
        sql_sofa_bilirubin = sql_file.read()
    sofa_bilirubin = read_sql(sql_sofa_bilirubin, con)

    # calculate SOFA liver score:
    print('Processing SOFA Liver...')
    sofa_bilirubin.loc[:, 'sofa_liver_score'] = 0
    sofa_bilirubin.loc[(sofa_bilirubin['value'] >= 20) & (sofa_bilirubin['value'] < 33), 'sofa_liver_score'] = 1
    sofa_bilirubin.loc[(sofa_bilirubin['value'] >= 33) & (sofa_bilirubin['value'] < 102), 'sofa_liver_score'] = 2
    sofa_bilirubin.loc[(sofa_bilirubin['value'] >= 102) & (sofa_bilirubin['value'] < 204), 'sofa_liver_score'] = 3
    sofa_bilirubin.loc[(sofa_bilirubin['value'] >= 204), 'sofa_liver_score'] = 4

    return sofa_bilirubin.sort_values(by=['admissionid', 'itemid', 'time']).reset_index(drop=True)


def get_sofa_cardiovascular_map(con):
    """
    Returns SOFA Mean arterial pressure score
    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    print('Querying SOFA Cardiovascular: MAP...')
    filename = './sql/common/mean_abp.sql'
    sql_filename = os.path.join(dirname, filename)
    with open(sql_filename, 'r') as sql_file:
        sql_mean_abp = sql_file.read()
    mean_abp = read_sql(sql_mean_abp, con)

    print('Processing SOFA Cardiovascular: MAP...')
    # remove extreme outliers, most likely data entry errors or measurement errors
    mean_abp.loc[(mean_abp['value'] > 165), 'value'] = np.NaN
    mean_abp.loc[(mean_abp['value'] <= 30), 'value'] = np.NaN
    mean_abp = mean_abp.dropna()

    # use mean_abp 'cleansed' dataframe
    sofa_cardiovascular_map = mean_abp.groupby(['admissionid', 'itemid', 'item']).agg(
        lowest_mean_abp=pd.NamedAgg(column='value', aggfunc='min')
    ).reset_index()

    # calculate SOFA cardiovascular score:
    sofa_cardiovascular_map.loc[:, 'sofa_cardiovascular_score'] = 0
    # MAP < 70
    sofa_cardiovascular_map.loc[(sofa_cardiovascular_map['lowest_mean_abp'] < 70), 'sofa_cardiovascular_score'] = 1

    return sofa_cardiovascular_map.sort_values(by=['admissionid']).reset_index(drop=True)


def get_sofa_cns(con) -> pd.DataFrame:
    """
    Return SOFA Central nervous system score

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    # get Glasgow Coma Scale-score
    print('Querying SOFA Central nervous system...')
    filename = './sql/common/gcs.sql'
    sql_filename = os.path.join(dirname, filename)
    with open(sql_filename, 'r') as sql_file:
        sql_gcs = sql_file.read()
    sofa_cns = read_sql(sql_gcs, con)

    print('Processing SOFA Central nervous system...')
    # calculate SOFA Central nervous system score:
    # the SOFA score assumes that the GCS is either determined without sedation, or was known before sedation was
    # started. Elective patients while being sedated are considered having GCS=15 with respect to the SOFA CNS score.
    #
    # Medical staff were trained to document an 'estimated/expected' GCS for sedated/intubated patients at least for the
    # first 24 hours, so should be the most reliable with respect to SOFA scoring

    # If scores have been performed by medical staff remove all others
    admissionids = sofa_cns[sofa_cns['registeredby'] == 'ICV_Medisch Staflid']['admissionid']
    sofa_cns = sofa_cns.drop(sofa_cns[
                                 (sofa_cns['admissionid'].isin(admissionids)) &
                                 ~(sofa_cns['registeredby'] == 'ICV_Medisch Staflid')
                                 ].index
                             )

    admissionids = sofa_cns[sofa_cns['registeredby'] == 'ICV_Medisch']['admissionid']
    sofa_cns = sofa_cns.drop(sofa_cns[
                                 (sofa_cns['admissionid'].isin(admissionids)) &
                                 ~(sofa_cns['registeredby'] == 'ICV_Medisch')
                                 ].index
                             )

    # The most common problem for SOFA scoring is documenting GCS of 3 while patients being sedated
    # (e.g. directly after surgery or after intubation)
    # Drop records with gcs_score = 3 AND sedated
    sofa_cns = sofa_cns.drop(sofa_cns[
                                 (sofa_cns['gcs_score'] == 3) & ~(sofa_cns['sedatives_given'].isna()) &
                                 (sofa_cns['registeredby'].isin(['ICV_IC-Verpleegkundig', 'ICV_MC-Verpleegkundig']))
                                 ].index
                             )

    # For GCS going from e.g. E1M1V1 to E3M6V5 within 24 hours
    sofa_cns[sofa_cns['gcs_score'] == 15][['admissionid', 'measuredat']]
    first_gcs15 = sofa_cns[sofa_cns['gcs_score'] == 15].groupby('admissionid').agg(
        first_gcs15_measuredat=('measuredat', 'first'))
    sofa_cns = sofa_cns.merge(first_gcs15, on='admissionid')
    sofa_cns = sofa_cns.drop(sofa_cns[
                                 (sofa_cns['gcs_score'] == 3) &
                                 (sofa_cns['measuredat'] < sofa_cns['first_gcs15_measuredat'])
                                 ].index
                             )
    # the GCS verbal (V) score could be documented as 'Intubated' (v_score = 0 or -2), often considered similar
    # to a value of 1. However, this often does not reflect the actual level of consciousness, especially if the
    # eyes and motor score values are not '1' as well.
    # First remove all score that do not contain at least a valid eyes and motor score
    sofa_cns = sofa_cns.drop(sofa_cns[
                                 (sofa_cns['eyes_score'].isna()) |
                                 (sofa_cns['motor_score'].isna())
                                 ].index
                             )
    # assume E1M1 also has V1
    sofa_cns.loc[(sofa_cns['eyes_score'] == 1) & (sofa_cns['motor_score'] == 1), 'verbal_score'] = 1

    # remove intubated scores including scores with E4M6V1 and perform back/forward fill
    sofa_cns.loc[(sofa_cns['verbal_score'] == -2), 'verbal_score'] = np.NaN
    sofa_cns.loc[(sofa_cns['verbal_score'] == 0), 'verbal_score'] = np.NaN
    sofa_cns.loc[(sofa_cns['eyes_score'] == 4) & (sofa_cns['motor_score'] == 6) & (sofa_cns['verbal_score'] == 1),
                 'verbal_score'] = np.NaN

    # assume for missing verbal scores that future measurements should contain the actual value, otherwise
    # forward fill for still missing values
    sofa_cns['verbal_score'] = sofa_cns.groupby(['admissionid'])['verbal_score'].bfill()
    sofa_cns['verbal_score'] = sofa_cns.groupby(['admissionid'])['verbal_score'].ffill()

    # recalculate total GCS score
    sofa_cns['gcs_score'] = sofa_cns['eyes_score'] + sofa_cns['motor_score'] + sofa_cns['verbal_score']

    # calculate SOFA GCS score based on 'corrected' GCS scores:
    sofa_cns.loc[:, 'sofa_cns_score'] = 0
    sofa_cns.loc[(sofa_cns['gcs_score'] >= 13) & (sofa_cns['gcs_score'] < 15), 'sofa_cns_score'] = 1
    sofa_cns.loc[(sofa_cns['gcs_score'] >= 10) & (sofa_cns['gcs_score'] < 13), 'sofa_cns_score'] = 2
    sofa_cns.loc[(sofa_cns['gcs_score'] >= 6) & (sofa_cns['gcs_score'] < 10), 'sofa_cns_score'] = 3
    sofa_cns.loc[(sofa_cns['gcs_score'] < 6), 'sofa_cns_score'] = 4

    return sofa_cns.sort_values(by=['admissionid', 'time']).reset_index(drop=True)


def get_sofa_renal_daily_urine_output(con) -> pd.DataFrame:
    """
    Return SOFA Urine output score

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    # get urine output
    print('Querying SOFA Renal: urine output...')
    filename = './sql/common/urine_output.sql'
    sql_filename = os.path.join(dirname, filename)
    with open(sql_filename, 'r') as sql_file:
        sql_sofa_renal_urine_output = sql_file.read()
    sofa_renal_urine_output = read_sql(sql_sofa_renal_urine_output, con)

    print('Processing SOFA Renal: urine output...')
    # probably decimal error when entering volumes > 2500
    sofa_renal_urine_output.loc[(sofa_renal_urine_output['value'] > 2500), 'value'] = sofa_renal_urine_output[
                                                                                          'value'] / 10
    # remove extreme outliers, most likely data entry error)
    sofa_renal_urine_output.loc[(sofa_renal_urine_output['value'] > 4500), 'value'] = np.NaN
    sofa_renal_urine_output = sofa_renal_urine_output.dropna()

    # get urine output per 24 hours
    sofa_renal_daily_urine_output = sofa_renal_urine_output.groupby(['admissionid']).agg(
        daily_urine_output=pd.NamedAgg(column='value', aggfunc='sum')
    ).reset_index()

    # calculate SOFA renal score for urine output:
    sofa_renal_daily_urine_output.loc[:, 'sofa_renal_score'] = 0

    # urine output < 500 ml/day
    sofa_renal_daily_urine_output.loc[(
                                          ((sofa_renal_daily_urine_output['daily_urine_output'] < 500) &
                                           (sofa_renal_daily_urine_output['daily_urine_output'] > 200))
                                      ), 'sofa_renal_score'] = 3

    # urine output < 200 ml/day
    sofa_renal_daily_urine_output.loc[(
                                          ((sofa_renal_daily_urine_output['daily_urine_output'] < 200))
                                      ), 'sofa_renal_score'] = 4
    return sofa_renal_daily_urine_output.sort_values(by=['admissionid']).reset_index(drop=True)


def get_sofa_renal_creatinine(con) -> pd.DataFrame:
    print('Querying SOFA Renal: creatinine...')
    # get serum creatinine
    filename = './sql/common/creatinine_acute_kidney_injury_failure.sql'
    sql_filename = os.path.join(dirname, filename)
    with open(sql_filename, 'r') as sql_file:
        sql_creatinine = sql_file.read()
    creatinine = read_sql(sql_creatinine, con)

    print('Processing SOFA Renal: creatinine...')
    # remove extreme outliers, most likely data entry errors (manual_entry = True)
    creatinine.loc[(creatinine['value'] < 30) & (creatinine['manual_entry'] is True), 'value'] = np.NaN
    creatinine = creatinine.dropna(subset=['value'])

    # get highest creatinine per 24 hours
    # use creatinine 'cleansed' dataframe from APACHE score
    sofa_renal_creatinine = creatinine.groupby(['admissionid']).agg(
        max_creatinine=pd.NamedAgg(column='value', aggfunc='max')
    ).reset_index()

    # calculate SOFA renal score for creatinine:
    sofa_renal_creatinine.loc[:, 'sofa_renal_score'] = 0

    # creatinine 110-170 umol/l
    sofa_renal_creatinine.loc[(
                                  ((sofa_renal_creatinine['max_creatinine'] >= 110) &
                                   (sofa_renal_creatinine['max_creatinine'] < 171))
                              ), 'sofa_renal_score'] = 1

    # creatinine 171-299 umol/l
    sofa_renal_creatinine.loc[(
                                  ((sofa_renal_creatinine['max_creatinine'] >= 171) &
                                   (sofa_renal_creatinine['max_creatinine'] < 300))
                              ), 'sofa_renal_score'] = 2

    # creatinine 300-440 umol/l
    sofa_renal_creatinine.loc[(
                                  ((sofa_renal_creatinine['max_creatinine'] >= 300) &
                                   (sofa_renal_creatinine['max_creatinine'] <= 440))
                              ), 'sofa_renal_score'] = 3

    # creatinine >440 umol/l
    sofa_renal_creatinine.loc[(
                                  ((sofa_renal_creatinine['max_creatinine'] > 440))
                              ), 'sofa_renal_score'] = 4

    return sofa_renal_creatinine.sort_values(by=['admissionid']).reset_index(drop=True)


def get_sofa_admission(con) -> pd.DataFrame:
    """
    Returns a dataframe containing SOFA score in the first 24 hours of ICU admission for all patients.
    See the [SOFA](https://github.com/AmsterdamUMC/AmsterdamUMCdb/blob/master/concepts/severityscores/sofa.ipynb)
    notebook for additional information on data processing and example data and plots.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    print('Starting SOFA at admission...')

    # loads the admissions table for stratification on admission unit
    print('Querying admissions...')
    sql_admission = "SELECT * FROM admissions;"
    admissions = read_sql(sql_admission, con)

    # SOFA Respiration: PaO2 / FiO2 (mmHg)
    sofa_respiration = get_sofa_respiration(con)

    # SOFA: Coagulation - Platelets (x10^3/mm^3)
    sofa_platelets = get_sofa_platelets(con)

    # SOFA: Liver - Bilirubin (µmol/l)
    sofa_bilirubin = get_sofa_bilirubin(con)

    # SOFA: Cardiovascular - Hypotension
    # cardiovascular medication: vasopressors and / or inotropes
    sofa_cardiovascular_meds = get_sofa_cardiovascular_meds(con)

    # Mean arterial pressure
    sofa_cardiovascular_map = get_sofa_cardiovascular_map(con)

    # combine the scores from MAP and cardiovascular medication
    sofa_cardiovascular = pd.concat([sofa_cardiovascular_map, sofa_cardiovascular_meds], sort=False).sort_values(
        by='admissionid')

    # SOFA: Central nervous system
    sofa_cns = get_sofa_cns(con)

    # SOFA: Renal - Creatinine or urine output
    # get urine output
    sofa_renal_daily_urine_output = get_sofa_renal_daily_urine_output(con)

    # serum creatinine
    sofa_renal_creatinine = get_sofa_renal_creatinine(con)

    # combine the scores from creatinine and urine output
    sofa_renal = pd.concat([sofa_renal_creatinine, sofa_renal_daily_urine_output], sort=False).sort_values(
        by='admissionid')

    # SOFA Score
    ############
    print('Merging SOFA Scores...')
    # merge the scores
    sofa = admissions['admissionid']

    # max respiration score
    scores = sofa_respiration.groupby('admissionid')['sofa_respiration_score'].max().to_frame(
        'sofa_respiration_score').sort_values(by=['admissionid']).reset_index()
    sofa = pd.merge(sofa, scores, on='admissionid', how='left')

    # max coagulation score
    scores = sofa_platelets.groupby('admissionid')['sofa_coagulation_score'].max().to_frame(
        'sofa_coagulation_score').sort_values(by=['admissionid']).reset_index()
    sofa = pd.merge(sofa, scores, on='admissionid', how='left')

    # max liver score
    scores = sofa_bilirubin.groupby('admissionid')['sofa_liver_score'].max().to_frame(
        'sofa_liver_score').sort_values(by=['admissionid']).reset_index()
    sofa = pd.merge(sofa, scores, on='admissionid', how='left')

    # max cardiovascular score
    scores = sofa_cardiovascular.groupby('admissionid')['sofa_cardiovascular_score'].max().to_frame(
        'sofa_cardiovascular_score').sort_values(by=['admissionid']).reset_index()
    sofa = pd.merge(sofa, scores, on='admissionid', how='left')

    # max central nervous system score
    scores = sofa_cns.groupby('admissionid')['sofa_cns_score'].max().to_frame(
        'sofa_cns_score').sort_values(by=['admissionid']).reset_index()
    sofa = pd.merge(sofa, scores, on='admissionid', how='left')

    # max renal score
    scores = sofa_renal.groupby('admissionid')['sofa_renal_score'].max().to_frame(
        'sofa_renal_score').sort_values(by=['admissionid']).reset_index()
    sofa = pd.merge(sofa, scores, on='admissionid', how='left')

    # calculate total score (add al values in columns)
    total_scores = sofa.set_index('admissionid').sum(axis=1, skipna=True).to_frame('sofa_total_score')
    sofa = pd.merge(sofa, total_scores, on='admissionid', how='left')

    # Add location and urgency for easier selection:
    sofa = pd.merge(sofa, admissions[['admissionid', 'location', 'urgency']], on='admissionid', how='left')
    # To force new array with new dtype (instead of inplace setting) (FutureWarning),
    # changed `sofa.loc[:, 'urgency'] = pd.to_numeric(sofa.loc[:, 'urgency'])` to:
    admissions['urgency'] = pd.to_numeric(admissions.loc[:, 'urgency'])

    print('SOFA processing complete.')
    return sofa.sort_values(by=['admissionid']).reset_index(drop=True)



import pandas as pd
import os
import re
from .scores import get_sofa_admission, get_sofa_cardiovascular_meds
from .util import read_sql, query

# surgical regular expressions
re_cardiosurg = r'(?:CABG|AVR|hartchirurgie|heart surgery|Chron. cardiovasculaire ziekte|hartkleppen' + \
                r'|cardiovascula|MVP|MVR|mitral|tricuspid|pericard|aortic.*valve|lobectom|segment|thorax|Bentall' \
                r'|aorta-ascendens|aorta-boog' + \
                r'|aorta-wortel|aorta-descendens|lung|pneumectomie|bullectom|respiratoir ' \
                r'neoplasm|thoracoscop|thoracotom(?:y|ie)|respirato' + \
                r'|vrije wand ruptuur|VSR|ASD|pleurectom|intracardiac|aneurysmectom|congenital defect repair)(?! for ' \
                r'esophag)'
re_neurosurg = r'neuro|tentorieel|cranial|subdur|epidur|subarachn|intracerbr|hoofdtrauma|SAB|S.A.H.|CNS|Hoofd|' \
               r'seizures' + \
               r'|craniotom|cranioplast|spinal|dwarslaesie|ventriculstom|transphenoidal|brain|A.V.M.|Arteriovenous ' \
               r'malformation' + \
               r'|brughoek|shunts and revisions|stereotactic|Burr hole|cerebrospinal'
re_vascsurg = r'vaatchirurgie|vasc.*surg|EVAR|aorta vervanging|perifeer vasc|embolectom|aneurysm|carotid|endovasc' + \
              r'|dissectie|endarterectom|thrombectomy|dilatation|PTCA|all other ' \
              r'bypass|femoral-popliteal|aorto-femoral|femoral-femoral' + \
              r'|transplantectom'
re_gisurg = r'oesophagus|esophageal|maag|stomach|darm|intestin|gastro-intestin|pancreatitis|laparotom|gastro' \
            r'-intestinale perforatie' + \
            r'|galblaas|Bleeding-.*GI|other GI|colon|rectal|GI.*surgery|GI ' \
            r'obstruction|Whipple|diverticular|appendectomy' + \
            r'|peritonitis|cholecystectomy|exenteration'
re_uro = r'(?<!ne)(?:urolog|cystectomy|genitourinary surgery|prostatectom|ileal\-conduit|orchiectomy|bladder repair' + \
         r'|nefrectom|nephrectom|renaal neopsplama)'
re_obgyn = r'hysterectom|Cesarean|ectopic pregnancy'
re_gensurg = r'mond/keel|cancer oral|cancer laryngeal|spondylodes|Fusion-spinal|devices for spine|orthopedic' + \
             r'|renaal|metabol|endocrin|thyroid|hip replacement|knee ' \
             r'replacement|adrenalectom|tracheostomy|bewaking|amputation|apnea-sleep' + \
             r'|lymph node dissection|mastectomy|cosmetic|fracture-pathological|bewaking'
re_trauma_surg = r'(?<!non-)(?<!see )(?:trauma|hypotherm|smoke inhalation)(?!, see trauma)(?! see)(?!: see)'
re_tx_surg = r'niertransplantatie|kidney transplant|renaal transplantatie|pancreastransplantatie'

re_respfailure_surg = r'resp.*insuff|na respiratoir arrest|arrest, respiratory|atelectas|pneumoni|na ok'

re_sepsis_surg = r'sepsis|pneumoni|GI perforation|perforation/rupture|infection|abscess|GI Vascular ' \
                 r'ischemia|diverticular|appendectomy|peritonitis'
re_cardiacarrest_surg = r'Cardiac arrest.*respiratory arrest|na reanimatie'
re_hepa = r'lever'
re_surg_other = r'diagnose anders|respiratoir|cardiovasculair|niet operatief'

re_surgical_medical = r'Bloeding tractus digestivus|Haemorragische shock|Gastro-intestinale bloeding|' \
                      r'Bleeding, upper GI' + \
                      r'|hematologisch|Hematologische maligniteit|Haematologisch|Renaal'

re_surgical = '(?:' + \
              re_cardiosurg + '|' + \
              re_neurosurg + '|' + \
              re_vascsurg + '|' + \
              re_gisurg + '|' + \
              re_uro + '|' + \
              re_obgyn + '|' + \
              re_gensurg + '|' + \
              re_trauma_surg + '|' + \
              re_tx_surg + '|' + \
              re_hepa + '|' + \
              re_surg_other + '|' + \
              re_surgical_medical + '|' + \
              re_respfailure_surg + '|' + \
              re_sepsis_surg + '|' + \
              re_cardiacarrest_surg + '|' + \
              ')'

# medical
re_respfailure_med = r'(?<! without )(?:resp.*insuff|pneumoni|respirato|luchtweg obstructie|obstruction-airway' + \
                     r'|chronisch obstructieve longziekte|emphysema|asthma|aspiratie|aspiration|longembolie|pulmonary' \
                     r'|pulmonaire' + \
                     r'|long|lung|atelectas|ALI/ARDS|ARDS|pleural|cancer, lung|pneumothorax|verdrinking|near ' \
                     r'drowning|weaning|hemothorax|PCP infect)'
re_cardiacarrest_med = r'Cardiac arrest.*respiratory arrest|na reanimatie'
re_sepsis_med = r'sepsis|septic|infect|pneumoni|cholangitis|pancr|endocarditis|meningitis|GI ' \
                r'perforation|abces|abscess|darm ischaemie|GI vascular|fasciitis' + \
                r'|inflammatory|peritonitis'
re_cardio = r'cardiogene shock|shock, cardiogenic|angina|ritme|rhythm|cardiovascular|cardiovasculair' + \
            r'|myocardial|endocarditis|coronair|coronary|cardiomyopath|tamponade|pericardial|papillary ' \
            r'muscle|^MI|hartkleppen' + \
            r'|CHF|hart falen|decompensatio cordis'
re_neuro = '(?<!see )(?:insult|seizure|CVA|observatie neurologische status|intracerebraal ' \
           'haematoom|intracranial|intracerebr|subdur|subarachno' + \
           r'|epidur|coma|neurologisch|neurologic|CZS|S\.A\.B\.|neurologie|hoofdtrauma|head|neuro|muscula|spinal' \
           r'|meningitis|encephalitis' + \
           r'|myasthenia|vaatspasme protocol|Guillian-Barre|encephalopath|musculoskeletal)(?!-see Neurological System)'
re_bleeding = r'bloeding tractus digestivus|gastro-intestinale bloeding|gastro-intestinaal|bleeding, ' \
              r'upper GI|bleeding, lower GI' + \
              r'|bleeding, GI|ulcer'
re_gi_med = r'ileus|GI medical|GI obstruction'
re_tox = r'intox|overdosis|overdose|toxicity|withdrawal|drug'
re_trauma_med = r'(?<!non-)(?<!see )(?:trauma|hypotherm|smoke inhalation)(?!, see trauma)(?! see)(?!: see)'
re_hemonc = r'malign|hematolog|cancer|bloeding|TTP|HUS|leukemi|pancytopen|coagulopath|anemia|neutropen|lymph|sickle'
re_endo_med = r'metabolisme|keto-acidose|diabetic|metabolic|endocrine|hypertens|acid-base|hypoglyc|thyroid'
re_shock_med = r'hypovolemische shock|shock|haemorr|hemorrha|anaphylaxis'
re_nefro_med = r'renaal|renal|tubulus|genitourinary|urolog|nefr'
re_hepa_med = r'lever|hepatic'
re_obgyn_med = r'obstetrie|postpartum|eclampsia'
re_vasc_med = r'aneurysm|vascular|cardiovascular medical|Thrombus,arterial|vascular medical|dissect|EVAR|embolectom'
re_mon_med = r'apnea, sleep|monitoring|bewaking|observatie'
re_tx_med = r'transplant'

re_medical = '(?:' + \
             re_respfailure_med + '|' + \
             re_cardiacarrest_med + '|' + \
             re_sepsis_med + '|' + \
             re_cardio + '|' + \
             re_neuro + '|' + \
             re_bleeding + '|' + \
             re_gi_med + '|' + \
             re_tox + '|' + \
             re_trauma_med + '|' + \
             re_hemonc + '|' + \
             re_endo_med + '|' + \
             re_shock_med + '|' + \
             re_nefro_med + '|' + \
             re_hepa_med + '|' + \
             re_obgyn_med + '|' + \
             re_vasc_med + '|' + \
             re_mon_med + '|' + \
             re_tx_med + '|' + \
             ')'


def get_surgical_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with a surgical reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        return combined_diagnoses[combined_diagnoses['surgical'] == 1]
    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/surgical.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)


def get_medical_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with a medical reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        return combined_diagnoses[combined_diagnoses['surgical'] == 0]
    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/medical.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)



def get_cardiothoracic_surgery_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with cardio-thoracic surgery as the primary reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        return combined_diagnoses[
            (combined_diagnoses['surgical'] == 1)
            & (combined_diagnoses['diagnosis'].str.contains(re_cardiosurg, na=False, flags=re.IGNORECASE))]
    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/cardiac_surgery.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)


def get_respiratory_failure_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with respiratory failure as the primary reason for admission. Please
    note that many patients that suffer from (septic) shock will require mechanical ventilation often due to secondary
    respiratory failure (e.g. due to acute respiratory distress syndrome)

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        respfailure = combined_diagnoses[
            (
                    (combined_diagnoses['surgical'] == 1)
                    & (combined_diagnoses['diagnosis'].str.contains(re_respfailure_surg,
                                                                    na=False,
                                                                    flags=re.IGNORECASE))
            ) | (
                    (combined_diagnoses['surgical'] == 0)
                    & (combined_diagnoses['diagnosis'].str.contains(re_respfailure_med,
                                                                    na=False,
                                                                    flags=re.IGNORECASE))
            )
            ]

        return respfailure
    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/respiratory_failure.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)



def get_neurosurgery_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with neurosurgery as the primary reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        neurosurg = combined_diagnoses[
            (combined_diagnoses['surgical'] == 1)
            & (combined_diagnoses['diagnosis'].str.contains(re_neurosurg, na=False, flags=re.IGNORECASE))
            ]

        return neurosurg
    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/neurosurgery.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)


def get_gastrointestinal_surgery_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with gastro-intestinal surgery as the primary reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        gisurg = combined_diagnoses[
            (combined_diagnoses['surgical'] == 1)
            & (combined_diagnoses['diagnosis'].str.contains(re_gisurg, na=False, flags=re.IGNORECASE))
            ]

        return gisurg
    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/gastro_intestinal_surgery.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)


def get_cardiac_arrest_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with cardiac arrest as the primary reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        cardiacarrest = combined_diagnoses[
            (
                    (combined_diagnoses['surgical'] == 1)
                    & (combined_diagnoses['diagnosis'].str.contains(re_cardiacarrest_surg, na=False, flags=re.IGNORECASE))
            ) | (
                    (combined_diagnoses['surgical'] == 0)
                    & (combined_diagnoses['diagnosis'].str.contains(re_cardiacarrest_med, na=False, flags=re.IGNORECASE))
            )
            ]

        return cardiacarrest
    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/cardiac_arrest.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)


def get_vascular_surgery_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with vascular surgery as the primary reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        vascsurg = combined_diagnoses[
            (combined_diagnoses['surgical'] == 1)
            & (combined_diagnoses['diagnosis'].str.contains(re_vascsurg, na=False, flags=re.IGNORECASE))
            # exclude cerebral aneurysms:
            & ~(combined_diagnoses['diagnosis_group'].str.contains('neuro', na=False, flags=re.IGNORECASE))
            ]

        return vascsurg

    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/vascular_surgery.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)


def get_trauma_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with trauma as the primary reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        trauma = combined_diagnoses[
            (
                    (combined_diagnoses['surgical'] == 1)
                    & (combined_diagnoses['diagnosis'].str.contains(re_trauma_surg, na=False, flags=re.IGNORECASE))
            ) | (
                    (combined_diagnoses['surgical'] == 0)
                    & (combined_diagnoses['diagnosis'].str.contains(re_trauma_med, na=False, flags=re.IGNORECASE))
            )
            ]

        return trauma

    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/trauma.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)


def get_neuro_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with a neurologic disorders as the primary reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        neuro = combined_diagnoses[
            (combined_diagnoses['surgical'] == 0)
            & (combined_diagnoses['diagnosis'].str.contains(re_neuro, na=False, flags=re.IGNORECASE))
            # exclude trauma cases
            & ~(combined_diagnoses['diagnosis'].str.contains(re_trauma_surg, na=False, flags=re.IGNORECASE))
            ]

        return neuro

    else:
        raise NotImplementedError("Work in progress. Function not yet available for OMOP CDM version.")


def get_cardio_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with a cardiac disorders as the primary reason for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        combined_diagnoses = get_reason_for_admission(con)
        cardio = combined_diagnoses[
            (combined_diagnoses['surgical'] == 0)
            & (combined_diagnoses['diagnosis'].str.contains(re_cardio, na=False, flags=re.IGNORECASE))
            ]

        return cardio

    else:
        raise NotImplementedError("Work in progress. Function not yet available for OMOP CDM version.")


def get_sepsis_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with a diagnosis of sepsis at admission.

    The [Sepsis-3 definition](https://jamanetwork-com.vu-nl.idm.oclc.org/journals/jama/fullarticle/2492881)
    requires, in addition to a presumed infection, a Sequential [Sepsis-related] Organ Failure Assessment
    (SOFA) score of 2 points or more.

    Operationalization:
    * presumed infection (with dysregulated host response), based on:
    - sepsis_at_admission flagged in the admission form by the attending physician
    - the admission diagnosis, medical or surgical, is considered a severe infection, e.g., gastro-intestinal
    perforation, cholangitis, meningitis
    - use of antibiotics that are not used prophylactically after surgery
    - use of antibiotics and cultures drawn within 6 hours of admission

    AND

    * organ dysfunction based on change in total SOFA score ≥2 points
    - baseline SOFA score assumed to be 0
    - uses data from first 24 hours of ICU admission

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        print('Creating cohort Sepsis-3 at admission (infection and SOFA score ≥2)...')
        infection = get_infection_patients(con)

        print('Selecting patients based on SOFA total score SOFA ≥2')
        sofa = get_sofa_admission(con)
        infection = pd.merge(infection, sofa[
            ['admissionid', 'sofa_respiration_score', 'sofa_coagulation_score',
             'sofa_liver_score', 'sofa_cardiovascular_score', 'sofa_cns_score',
             'sofa_renal_score', 'sofa_total_score']], on='admissionid', how='left')

        sepsis = infection[infection['sofa_total_score'] >= 2]
        return sepsis
    else:
        raise NotImplementedError("Work in progress. Function not yet available for OMOP CDM version.")


def get_infection_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing patients with a (suspected) infection as the primary reason for admission.
    Similar to the Sepsis-3 cohort patients, but without using SOFA to select patients.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        print('Querying reason for admission...')
        combined_diagnoses = get_reason_for_admission(con)
        print('Selecting patients with presumed infection...')
        infection = combined_diagnoses[
            (
                    (
                        # use reasons for admission
                        # surgical admissions with sepsis
                            (combined_diagnoses['surgical'] == 1)
                            & (combined_diagnoses['diagnosis'].str.contains(re_sepsis_surg, na=False, flags=re.IGNORECASE))
                    ) | (
                        # medical admissions with sepsis
                            (combined_diagnoses['surgical'] == 0)
                            & (combined_diagnoses['diagnosis'].str.contains(re_sepsis_med, na=False, flags=re.IGNORECASE))
                    ) | (
                        # uses documentation at admission form (Early Goal Directed Therapy)
                        (combined_diagnoses['sepsis_at_admission'] == 1)
                    ) | (
                        # uses administered (therapeutic) antibiotics for determining sepsis
                        (combined_diagnoses['sepsis_antibiotics_bool'] == 1)
                    ) | (
                        # uses combination of administered antibiotics (that sometimes are used as prophylaxis) AND
                        # drawn cultures for determining sepsis
                            (combined_diagnoses['other_antibiotics_bool'] == 1)
                            & (combined_diagnoses['sepsis_cultures_bool'] == 1)
                    )
            ) & ~((combined_diagnoses['sepsis_at_admission'] == 0).fillna(False))
            # exclude all diagnoses where explicitly 'no sepsis' was documented, forces comparing `pd.NA`
            # to be considered False
            ]
        return infection

    else:
        # gets the SQL source file
        dirname = os.path.dirname(os.path.abspath(__file__))
        filename = './sql/diagnosis/infection.sql'
        sql_filename = os.path.join(dirname, filename)

        with open(sql_filename, 'r') as file:
            sql = file.read()
        return read_sql(sql, con)


def get_shock_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing all patients that present with or develop shock within 24 hours of admission.
    Definition:
    - Vasopressor or inotrope administered, AND
    - Lactate >= 2.0 mmol/l

        Arguments:
            con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        print('Creating cohort shock at admission/first 24 hours (vasopressors/inotropes administered '
              'and lactate ≥2 mmol/l)...')
        print('Querying reason for admission...')
        combined_diagnoses = get_reason_for_admission(con)

        # re-use SOFA cardiovascular score for vasopressors and/or inotropes
        sofa_cardiovascular_meds = get_sofa_cardiovascular_meds(con)
        lactate = query('common/lactate.sql', con)

        # max cardiovascular score based on vasopressors and inotropes
        max_cardiovascular_score = sofa_cardiovascular_meds.groupby('admissionid')[
            'sofa_cardiovascular_score'].max().to_frame(
            'sofa_cardiovascular_score').sort_values(by=['admissionid']).reset_index()
        shock = pd.merge(combined_diagnoses, max_cardiovascular_score, on='admissionid', how='left')

        # max lactate
        lactate_max = lactate.groupby('admissionid')['value'].max().to_frame(
            'lactate').sort_values(by=['admissionid']).reset_index()
        shock = pd.merge(shock, lactate_max, on='admissionid', how='left')

        # only keep patients with vasopressors and/or inotrope support (2 or higher), AND with lactate > 2.0 mmol/l
        print('Selecting patients with vasopressors and/or inotrope support (SOFA cardiovascular score 2 or higher), '
              'AND with lactate >= 2.0 mmol/l...')
        shock = shock[(shock['sofa_cardiovascular_score'] >= 2) & (shock['lactate'] >= 2.0)]
        return shock

    else:
        raise NotImplementedError("Work in progress. Function not yet available for OMOP CDM version.")


def get_invasive_ventilation_patients(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing all patients that were mechanically ventilated using an invasive mode.
    Definition:
    - Mechanical ventilation mode compatible with invasive ventilation.

        Arguments:
            con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    if legacy:
        print('Creating cohort invasive mechanical ventilation during ICU admission...')
        print('Querying reason for admission...')
        combined_diagnoses = get_reason_for_admission(con)

        # use mechanical ventilation query
        print('Querying mechanical ventilation (including possible non-invasive ventilation)...')
        ventilation = query('lifesupport/mechanical_ventilation.sql', con)

        # merge dataframes
        ventilation = pd.merge(combined_diagnoses, ventilation, on='admissionid', how='left')

        print('Selecting patients with invasive ventilation...')
        ventilation = ventilation[ventilation['invasive_bool'].fillna(False)]

        return ventilation

    else:
        raise NotImplementedError("Work in progress. Function not yet available for OMOP CDM version.")


def get_reason_for_admission(con, legacy=False) -> pd.DataFrame:
    """Returns a dataframe containing all patients with reasons for admission.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """

    # gets the SQL source file
    dirname = os.path.dirname(os.path.abspath(__file__))
    if legacy:
        filename = './sql/diagnosis/legacy/reason_for_admission.sql'
    else:
        filename = './sql/diagnosis/reason_for_admission.sql'
    sql_filename = os.path.join(dirname, filename)

    with open(sql_filename, 'r') as file:
        sql = file.read()
    result = read_sql(sql, con)

    return result



"""
# Test script for AmsterdamUMCdb using both PostgreSQL and Google BigQuery
Tests SOFA functions

## Requirements
### Databases
- Working PostgreSQL server with fully setup AmsterdamUMCdb
- Access to AmsterdamUMCdb instance on Google BigQuery

### Config files:
- config.ini, by copying the config.SAMPLE.ini file and adding the credentials
- credentials.json, created by following the [Quickstart: Using client libraries](https://cloud.google.com/bigquery/docs/quickstarts/quickstart-client-libraries)
"""

import os
import unittest

from amsterdamumcdb import connections
from amsterdamumcdb import scores
import pandas as pd
import pandas.testing as pd_testing


def syncDataTypes(dataframe_left, dataframe_right):
    for col in dataframe_left.columns:
        dtype_left = dataframe_left[col].dtype
        dtype_right = dataframe_right[col].dtype
        if dtype_left != dtype_right:
            dataframe_right[col] = dataframe_right[col].astype(dtype_left)


class SOFATests(unittest.TestCase):
    def assertDataframeEqual(self, a, b, msg):
        try:
            syncDataTypes(a, b)
            pd_testing.assert_frame_equal(a, b)
        except AssertionError as e:
            raise self.failureException(msg) from e

    def setUp(self):
        config = 'config.ini'
        if not os.path.exists(config):
            config = os.path.join('../', config)

        self.config_gbq = connections.connect_gbq(config)
        self.con_pg = connections.connect_pg(config)

        self.addTypeEqualityFunc(pd.DataFrame, self.assertDataframeEqual)

    def test_sofa_admission(self):
        print('Testing retrieving full SOFA score at admission...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_admission(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_admission(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)

    def test_sofa_bilirubin(self):
        print('Testing SOFA Liver: bilirubin...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_bilirubin(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_bilirubin(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)

    def test_sofa_cardiovascular_map(self):
        print('Testing SOFA Cardiovascular: MAP...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_cardiovascular_map(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_cardiovascular_map(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)

    def test_sofa_cardiovascular_meds(self):
        print('Testing SOFA Cardiovascular: vasopressors/inotropes...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_cardiovascular_meds(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_cardiovascular_meds(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)

    def test_sofa_cns(self):
        print('Testing SOFA Central Nervous System: GCS...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_cns(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_cns(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)

    def test_sofa_platelets(self):
        print('Testing SOFA Coagulation: Platelets...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_platelets(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_platelets(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)

    def test_sofa_renal_creatinine(self):
        print('Testing SOFA Renal: Creatinine...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_renal_creatinine(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_renal_creatinine(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)

    def test_sofa_renal_urine(self):
        print('Testing SOFA Renal: Daily urine output...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_renal_daily_urine_output(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_renal_daily_urine_output(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)

    def test_sofa_respiration(self):
        print('Testing SOFA Respiration: Oxygenation...')
        print('Google BigQuery...')
        sofa_gbq = scores.get_sofa_respiration(con=self.config_gbq)
        print(sofa_gbq)

        print('PostgreSQL...')
        sofa_pg = scores.get_sofa_respiration(con=self.con_pg)
        print(sofa_pg)

        self.assertEqual(sofa_gbq, sofa_pg)


if __name__ == '__main__':
    unittest.main()



"""
# Test script for AmsterdamUMCdb using both PostgreSQL and Google BigQuery
Tests utils for working with SQL queries between the two database systems

## Requirements
### Databases
- Working PostgreSQL server with fully setup AmsterdamUMCdb
- Access to AmsterdamUMCdb instance on Google BigQuery

### Config files:
- config.ini, by copying the config.SAMPLE.ini file and adding the credentials
- credentials.json, created by following the [Quickstart: Using client libraries](https://cloud.google.com/bigquery/docs/quickstarts/quickstart-client-libraries)
"""

import os
import pandas as pd
import configparser
import unittest
import amsterdamumcdb as adb

from google.cloud import bigquery
import psycopg2

from amsterdamumcdb import connections
from amsterdamumcdb import cohorts
from amsterdamumcdb import util


class UtilTests(unittest.TestCase):
    config = 'config.ini'
    if not os.path.exists(config):
        config = os.path.join('../', config)

    config_gbq = connections.connect_gbq(config)
    con_pg = connections.connect_pg(config)

    def test_sql_validated(self):
        orig_sql = "SELECT COUNT(*) FROM numericitems"
        print(f'Original SQL statement for validated data:\n{orig_sql}')

        sql_gbq = util.select_validated(orig_sql, con=self.config_gbq)
        print(f'Transformed SQL statement for Google BigQuery:\n{sql_gbq}')
        gbq = util.read_sql(orig_sql, con=self.config_gbq)
        print(gbq)

        sql_pg = util.select_validated(orig_sql, con=self.con_pg)
        print(f'Transformed SQL statement for PostgreSQL:\n{sql_pg}')
        pg = util.read_sql(orig_sql, con=self.con_pg)
        print(pg)

        self.assertEqual(gbq.values[0][0], pg.values[0][0])
        self.assertEqual(gbq.values[0][0], 68672818)  # number of validated numericitems excluding unvalidated

    def test_sql_include_unvalidated(self):
        orig_sql = "SELECT COUNT(*) FROM numericitems"
        print(f'Original SQL statement for unvalidated data:\n{orig_sql}')

        sql_gbq = util.select_validated(orig_sql, con=self.config_gbq, include_unvalidated=True)
        print(f'Transformed SQL statement for Google BigQuery:\n{sql_gbq}')
        gbq = util.read_sql("SELECT COUNT(*) FROM numericitems", con=self.config_gbq, include_unvalidated=True)
        print(gbq)

        sql_pg = util.select_validated(orig_sql, con=self.con_pg, include_unvalidated=True)
        print(f'Transformed SQL statement for PostgreSQL:\n{sql_pg}')
        pg = util.read_sql("SELECT COUNT(*) FROM numericitems", con=self.con_pg, include_unvalidated=True)
        print(pg)

        self.assertEqual(gbq.values[0][0], pg.values[0][0])
        self.assertEqual(gbq.values[0][0], 977625612)  # total number of numericitems including unvalidated


if __name__ == '__main__':
    unittest.main()






import pandas as pd
import os
import re

import sqlalchemy

from .bigquery import pg2bq



def pg2bq(sql):
    """
    Translate a SQL query string from PostgreSQL dialect to Big Query SQL dialect
    Please note that this has only been tested with queries from this repository so your mileage may vary.

    Returns:
        dataframe:
    """

    # translate SPLIT_PART to SPLIT
    """
    PostgreSQL:
    split_part(string text, delimiter text, field int)
    Split string on delimiter and return the given field (counting from one)

    BigQuery: 
    SPLIT(value[, delimiter])
    Splits value using the delimiter argument. Return type: ARRAY of type STRING or ARRAY of type BYTES
    """
    re_pg_split_part = r'split_part\(\s*(.*)\s*,\s*(.*)\s*,\s*([0-9]*)\s*\)'
    re_bq_split = r'SPLIT(\1, \2)[SAFE_ORDINAL(\3)]'
    sql_translated = re.sub(re_pg_split_part, re_bq_split, sql)

    # translate ILIKE to LIKE
    """"
    PostgreSQL: 
    The key word ILIKE can be used instead of LIKE to make the match case-insensitive according to the active locale. 
    This is not in the SQL standard but is a PostgreSQL extension.
    """""
    re_pg_ilike = r'(?P<string>[^\(\s]+)\s+(?P<not>NOT\s+)?(?P<operator>ILIKE)\s+(?P<pattern>(?:(?P<quote>[\"\'])' \
                  r'(?:.)*?(?P=quote))|(?:[^\)\s]+))'
    re_bq_like = r'UPPER(\g<string>) \g<not>LIKE UPPER(\g<pattern>)'
    sql_translated = re.sub(re_pg_ilike, re_bq_like, sql_translated)

    # translate Byte to integer
    # after exporting to csv: bytes will be converted into integers
    re_pg_byte = r'(?P<column>[^\(\s]+)\s+(?P<operator>=)\s+(?P<byte>B\'(?P<value>[0-1])\')'
    re_bq_int = r'\g<column> \g<operator> \g<value>'
    sql_translated = re.sub(re_pg_byte, re_bq_int, sql_translated)

    # translate INTERVAL
    re_pg_interval = r'(?P<interval>INTERVAL)\s+\'(?P<quantity>[0-9]+)\s+(?P<unit>\w+)\''
    re_bg_interval = r'\g<interval> \g<quantity> \g<unit>'
    sql_translated = re.sub(re_pg_interval, re_bg_interval, sql_translated)
    return sql_translated


def union_tables(match_object):
    """
    Replaces FROM/JOIN statement of numericitems/listitems with UNION ALL to retrieve all unvalidated records
    """
    command = match_object.group('command')
    table = match_object.group('table')
    alias = match_object.group('alias')

    if len(alias) > 0:
        return f'{command} (SELECT * FROM {table} UNION ALL SELECT * FROM {table}_unvalidated) {alias} '
    else:
        return f'{command} (SELECT * FROM {table} UNION ALL SELECT * FROM {table}_unvalidated) {table} '


def exclude_unvalidated(match_object):
    """
    Excluded unvalidated data from the numericitems and listitems table by adding `WHERE NOT registeredby IS NULL`
    to the respective FROM/JOIN table statements.
    """
    command = match_object.group('command')
    table = match_object.group('table')
    alias = match_object.group('alias')

    if len(alias) > 0:
        return f'{command} (SELECT * FROM {table} WHERE NOT registeredby IS NULL) {alias} '
    else:
        return f'{command} (SELECT * FROM {table} WHERE NOT registeredby IS NULL) {table} '


def select_validated(sql, con, include_unvalidated=False):
    """
    Translate query by either including or excluding unvalidated (high volume) data
    When using BigQuery:
    For numericitems UNION ALL numericitems_unvalidated, and
    For lisitems UNION ALL listitems_unvalidated

    Args:
        sql: SQL statement
        con: psycopg2 connection or pandas-gbq Google BigQuery config
        include_unvalidated: If True will include the unvalidated data (Default: False)

    Returns: Transformed SQL statement
    """

    re_tables = r'(?P<command>FROM|JOIN)\s*(?P<table>numericitems|listitems)\s*(?:AS\s*)?(?!RIGHT|LEFT|FULL|' \
                r'INNER|OUTER|JOIN|UNION|WHERE|GROUP|ON|ORDER|HAVING|LIMIT)(?P<alias>\S*)'

    if include_unvalidated:
        if type(con) is dict: # BigQuery config
            # UNION ALL regular and unvalidated tables
            sql_translated = re.sub(re_tables, union_tables, sql)
            return sql_translated
        else:
            # local databases already contain all values in the numericitems and listitems tables
            return sql
    else:
        if type(con) is dict: # BigQuery config
            # BigQuery already has unvalidated data removed from main tables
            return sql
        else:
            # remove unvalidated data from SQL query
            sql_translated = re.sub(re_tables, exclude_unvalidated, sql)
            return sql_translated


def read_sql(sql, con, include_unvalidated=False):
    """
    Small wrapper around Pandas read_sql and read_gbq to de-duplicate code fragments that should work both on (local)
    postgreSQL instances and Google BigQuery ofr AmsterdamUMCdb.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """

    if type(con) is dict:  # BigQuery config
        # translate query from PostgreSQL dialect to BigQuery
        sql = pg2bq(sql)
        sql = select_validated(sql=sql, con=con, include_unvalidated=include_unvalidated)
        result = pd.read_gbq(sql, configuration=con, use_bqstorage_api=True)
    else:
        sql = select_validated(sql=sql, con=con, include_unvalidated=include_unvalidated)
        # Escape SQL for use in SQLAlchemy to prevent 'TypeError: dict is not a sequence' when using `%`
        result = pd.read_sql(sqlalchemy.text(sql), con)

    return result


def query(filename, con):
    """
    Queries using a SQL script located in the `sql` folder. Translates the SQL code as needed.

    Arguments:
        con -- psycopg2 connection or pandas-gbq Google BigQuery config
    """
    # gets the SQL source file
    dirname = os.path.dirname(os.path.abspath(__file__))
    sql_filename = os.path.join(dirname, 'sql', filename)

    print(f"Querying '{sql_filename}'...")
    with open(sql_filename, 'r') as sql_file:
        sql = sql_file.read()

    return read_sql(sql, con)



"""
# Test script for AmsterdamUMCdb using both PostgreSQL and Google BigQuery
Tests cohort scripts for working with SQL queries between the two database systems

## Requirements
### Databases
- Working PostgreSQL server with fully setup AmsterdamUMCdb
- Access to AmsterdamUMCdb instance on Google BigQuery

### Config files:
- config.ini, by copying the config.SAMPLE.ini file and adding the credentials
- credentials.json, created by following the [Quickstart: Using client libraries](https://cloud.google.com/bigquery/docs/quickstarts/quickstart-client-libraries)
"""
import os
import pandas as pd
import configparser
import unittest
import amsterdamumcdb as adb

from google.cloud import bigquery
import psycopg2

from amsterdamumcdb import connections
from amsterdamumcdb import cohorts


class CohortsTests(unittest.TestCase):
    config = 'config.ini'
    if not os.path.exists(config):
        config = os.path.join('../', config)

    config_gbq = connections.connect_gbq(config)
    con_pg = connections.connect_pg(config)

    def test_cardiothoracic_surgery_cohort(self):
        print('Testing cardio-thoracic surgery cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_cardiothoracic_surgery_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_cardiothoracic_surgery_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(5938, count_gbq)

    def test_infection_cohort(self):
        print('Testing infection cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_infection_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_infection_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(3152, count_gbq)

    def test_respiratory_failure_cohort(self):
        print('Testing respiratory failure cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_respiratory_failure_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_respiratory_failure_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(1517, count_gbq)

    def test_neurosurgery_cohort(self):
        print('Testing neurosurgery cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_neurosurgery_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_neurosurgery_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(1618, count_gbq)

    def test_gastrointestinal_surgery_cohort(self):
        print('Testing gastrointestinal surgery cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_gastrointestinal_surgery_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_gastrointestinal_surgery_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(1149, count_gbq)

    def test_cardiac_arrest_cohort(self):
        print('Testing cardiac arrest cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_cardiac_arrest_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_cardiac_arrest_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(961, count_gbq)

    def test_vascular_surgery_cohort(self):
        print('Testing vascular surgery cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_vascular_surgery_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_vascular_surgery_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(1037, count_gbq)

    def test_trauma_cohort(self):
        print('Testing trauma cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_trauma_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_trauma_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(1242, count_gbq)

    def test_neuro_cohort(self):
        print('Testing neurologic disorders cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_neuro_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_neuro_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(627, count_gbq)

    def test_cardio_cohort(self):
        print('Testing cardiac disorders cohort...')
        print('Google BigQuery...')
        patients_gbq = cohorts.get_cardio_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = cohorts.get_cardio_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(538, count_gbq)

    # @unittest.skip("Skip Sepsis-3 cohort testing") # this test will take some time
    def test_sepsis3_cohort(self):
        print('Testing Sepsis-3 cohort...')
        print('Google BigQuery...')
        patients_gbq = adb.get_sepsis_patients(con=self.config_gbq)
        print(patients_gbq)
        count_gbq = len(patients_gbq)
        print(f'{count_gbq} patients')

        print('PostgreSQL...')
        patients_pg = adb.get_sepsis_patients(con=self.con_pg)
        print(patients_pg)
        count_pg = len(patients_pg)
        print(f'{count_pg} patients')

        self.assertEqual(count_gbq, count_pg)
        # TODO: self.assertEqual(count_gbq, 3019)


if __name__ == '__main__':
    unittest.main()



import os
import configparser

import sqlalchemy
from google.cloud import bigquery
import psycopg2


def connect_gbq(config_file='config.ini'):
    """
    Setup connection for Google BigQuery using pandas gbq API

    Returns: config_gbq Dict
    """
    config = configparser.ConfigParser()
    config.read(config_file)

    credentials_file = os.path.join(os.path.dirname(os.path.abspath(config_file)),
                                    config['bigquery']['credentials_file'])

    os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = credentials_file
    # sets default dataset for AmsterdamUMCdb
    DATASET_PROJECT_ID = config['bigquery']['dataset_project_id']
    DATASET_ID = config['bigquery']['dataset_id']
    LOCATION = config['bigquery']['location']

    # create a Client object that handles authentication
    client = bigquery.Client()

    config_gbq = {'query':
        {'defaultDataset': {
            "datasetId": DATASET_ID,
            "projectId": DATASET_PROJECT_ID
        },
            'Location': LOCATION}
    }
    return config_gbq


def connect_pg(config_file='config.ini'):
    """
    Setup connection for SQLAlchemy for postgres using psycopg2.
    Currently, only a connection using the SQLAlchemy engine is supported.

    Returns: psycopg2 connection
    """
    import configparser
    import os
    config = configparser.ConfigParser()

    config.read(config_file)

    # Open a connection to the postgres database:
    pg_con = psycopg2.connect(database=config['psycopg2']['database'],
                              user=config['psycopg2']['username'], password=config['psycopg2']['password'],
                              host=config['psycopg2']['host'], port=config['psycopg2']['port'])
    pg_con.set_client_encoding('WIN1252')  # Uses code page for Dutch accented characters.
    pg_con.set_session(autocommit=True)

    cursor = pg_con.cursor()
    schema = config['psycopg2']['schema']
    cursor.execute(f'SET SCHEMA \'{schema}\';')  # set search_path to amsterdamumcdb schema

    # create SQLAlchemy engine for official pandas database support
    engine = sqlalchemy.create_engine('postgresql+psycopg2://', creator=lambda: pg_con)
    con = engine.connect()

    return con



# __init__.py
from .fluidbalance import get_fluidbalance
from .plotting import outliers_histogram
from .dictionary import get_dictionary
from .bigquery import pg2bq
from .cohorts import *
from .scores import *
